<?php

interface Rtgc_filtrable
{

    /**
     * Generate filter HTML
     * @param string $name
     * @param string $value
     * @return string
     */
    public function filter($name, $value);

    /**
     * Filtering data
     * @param string $column
     * @param string $value
     * @return void
     */
    public function filter_by($column, $value);
}
